from statelogic import *
from world import *
import random

class GameEntity(object):
    def __init__(self, world, name, image, initial_position):

        self.world = world
        self.name = name
        self.image = image
        self.location = initial_position
        self.destination = (0,0)
        self.speed = 0.

        self.brain = StateMachine()
        self.id = 0

    def render(self, surface):
        x,y = self.location
        w, h = self.image.get_size()
        surface.blit(self.image, (x-w/2, y-h/2))

    def process(self, time_passed):
        self.brain.think()

        if self.speed > 0 and self.location != self.destination:
            vec_to_destination = Vector.vector_from_points(self.location, self.destination)
            distance_to_destination = vec_to_destination.get_magnitude()
            heading = vec_to_destination.normalize()

            travel_distance = min(distance_to_destination, time_passed* self.speed)
            self.location = (self.location [0] + travel_distance * heading.x, self.location[1] + travel_distance * heading.y)




class Ant(GameEntity):

    def __init__(self, world, image, initial_position):
        GameEntity.__init__(self, world, "ant", image, initial_position)

        #add states
        exploring_state = AntStateExploring(self)
        seeking_state = AntStateSeeking(self)
        delivering_state = AntStateDelivering(self)
        #add more...


        #add states to state machine
        self.brain.add_state(exploring_state)
        self.brain.add_state(seeking_state)
        self.brain.add_state(delivering_state)

        self.carry_image = None

    def carry(self, image):
        self.carry_image = image

    def drop(self, surface):
        if self.carry_image:
            surface.blit(self.carry_image, self.location)
            self.carry_image = None

    def render(self, surface):
        GameEntity.render(self, surface)

        if self.carry_image:
            x, y = self.location
            w, h = self.carry_image.get_size()
            surface.blit(self.carry_image, (x-w, y-h/2))



class AntStateExploring(State):
    def __init__(self, ant):
        State.__init__(self, "exploring")
        self.ant = ant

    def random_destination(self):
        w, h = SCREEN_SIZE
        self.ant.destination = (random.randint(0,w), random.randint(0,h))
        

    def do_actions(self):
        #change direction in approx. every 20th call
        if random.randint(1, 20) == 1:
            self.random_destination()

    def check_conditions(self):
        leaf = self.ant.world.get_close_entity("leaf", self.ant.location)
        if leaf is not None:
            self.ant.leaf_id = leaf.id
            return "seeking"
        return None

    def entry_actions(self):
        self.ant.speed = 120. + random.randint(-30,30)
        self.random_destination()

class AntStateSeeking(State):
    def __init__(self, ant):
        State.__init__(self, "seeking")
        self.ant = ant


    def check_conditions(self):
        #check if leaf still exists
        leaf =  self.ant.world.get(self.ant.leaf_id)
        if leaf is None:
            return "exploring"

        if self.ant.location == self.ant.destination:
            self.ant.carry(leaf.image)
            #delete leaf
            self.ant.world.remove_entity(leaf)
            #enter delivering state
            return "delivering"

    def entry_actions(self):
        self.ant.destination = (self.ant.world.get(self.ant.leaf_id)).location

class AntStateDelivering(State):
    def __init__(self, ant):
        State.__init__(self, "delivering")
        self.ant = ant

    def entry_actions(self):
        self.ant.destination = NEST_POSITION

    def check_conditions(self):
        if self.ant.location == self.ant.destination:
            self.ant.drop(self.ant.world.background)
            return "exploring"

class Leaf(GameEntity):
    def __init__(self, world, image, initial_position):
        GameEntity.__init__(self, world, "leaf", image, initial_position)